use Test::More tests => 108;
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::DataCache::API::Cache qw ( UpdateLocalConnection );
use DE_EPAGES::Database::API::Connection qw ( ConnectTestSite DisconnectSite GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject JoinObjectPath ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );

local $ENV{'EPAGES_TESTSITE'} = 'Store' unless $ENV{'EPAGES_TESTSITE'};
my $ROOTPATH = 'Test';

sub Main {

    ConnectTestSite();

    # Setup database; usually each db section should contain an "key" entry where the aes can be found, like:
    #
    #   [Site.Default]
    #   datasource=dbi:mysql:mysql_socket=/var/lib/mysql/mysql.sock;database=storedb
    #   login=usr_storedb
    #   passwd=epages
    #   active=1
    #   key=hst58a42hvbt8671loegysq87n4mc7z1
    #
    # if not, we add one key here.
    my $dbi = GetCurrentDBHandle();
    $dbi->{'key'} = 'hst58a42hvbt8671loegysq87n4mc7z1' unless $dbi->{'key'};

    # create test classes
    ok( Init(), 'initialization' );

    my $Shop   = LoadObjectByPath(JoinObjectPath(['', $ROOTPATH, 'TestShop1' ]));
    ok( $Shop->id, 'create shop ' . $Shop->id );

    my $CustomerNo = GetNextCustomerNo($Shop);
    ok( length $CustomerNo >= 4, 'customer no. starts with 1000' );

    my $CustomerFolder = GetCustomerFolder($Shop);
    ok( !$CustomerFolder->existsChild($CustomerNo), 'customer no. does not exists yet' );

    my $Customer = $Shop->get('CustomerClass')->insertObject( {
            Parent => $CustomerFolder,
            Alias  => $CustomerNo,
        }
    );
    ok( $Customer->id, 'insert new customer' );
    ok( $Customer->alias eq $CustomerNo, 'Customer.Alias' );

    my $OrderNo = GetNextOrderNo($Shop);
    ok( length $OrderNo >= 4, 'order no. starts with 1000' );

    my $Order = $Customer->folder('Orders')->insertChild(
        'CustomerOrder', {
            'Alias'    => $OrderNo,
            'Shop'     => $Shop,
            'Customer' => $Customer,
        }
    );

    # add a line item container
    my $Container = $Order->insertChild(
        'LineItemContainer', {
            'Alias'      => 'LineItemContainer',
            'LanguageID' => $Shop->get('LanguageID'),
            'LocaleID'   => $Shop->get('LocaleID'),
            'CurrencyID' => $Shop->get('CurrencyID'),
            'TaxArea'    => $Shop->get('TaxMatrix')->defaultTaxArea,
            'TaxModel'   => $Shop->get('TaxModel'),
        }
    );
    $Order->set( { 'LineItemContainer' => $Container } );

    use constant SHORT_ENCRYPTED_LENGTH => 64;
    use constant NUMBER_RANGE => [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ];
    foreach my $TryNumber ( @{+NUMBER_RANGE} ) {
        my $LineItemPaymentDCC = $Order->insertChild(
            'LineItemPaymentDCC', {
                'Alias'             => '08/15 ' . $TryNumber,
                'CreditCardType'    => 'Visa',
                'CreditCardOwner'   => 'John Doe',
                'CreditCardNumber'  => '5555764423454545',
                'ValidFromMonth'    => 10,
                'ValidFromYear'     => 2000,
                'ValidThruMonth'    => 10,
                'ValidThruYear'     => 2020,
                'CVV'               => '123',
                'LineItemContainer' => $Container,
            }
        );

        # EPG-15717: don't get value from cache (for every second try)
        UpdateLocalConnection( $dbi->cacheKey ) if 0 == $TryNumber % 2;

        # Get methods should decrypt the data transparently.
        is( $LineItemPaymentDCC->get('CreditCardNumber'), '5555764423454545',
            "Creditcardnumber $TryNumber" );
        is( $LineItemPaymentDCC->get('CVV'), '123', "CVV $TryNumber" );

        # Get the encrypted data from DB directly.
        my $SQL = 'SELECT creditcardnumber, cvv FROM lineitempaymentdcc WHERE lineitempaymentdccid = ?';
        my $aaRows = GetCurrentDBHandle()->execute( $SQL, [ $LineItemPaymentDCC->id ] );

        is( length $aaRows->[0]->[0], SHORT_ENCRYPTED_LENGTH, "Creditcardnumber encrypted $TryNumber" );
        is( length $aaRows->[0]->[1], SHORT_ENCRYPTED_LENGTH, "Cvv encrypted $TryNumber" );

        # Update test
        # Get methods should decrypt the data transparently.
        $LineItemPaymentDCC->set({'CreditCardNumber' => '1111764423454545'});
        is( $LineItemPaymentDCC->get('CreditCardNumber'), '1111764423454545',
            'Creditcardnumber Update');

        # all non-number digits should be cut out
        $LineItemPaymentDCC->set({'CreditCardNumber' => '1223-3456-1234-9876'});
        is( $LineItemPaymentDCC->get('CreditCardNumber'), '1223345612349876',
            'Creditcardnumber Update - cutting out non-numbers (1)');
        $LineItemPaymentDCC->set({'CreditCardNumber' => '1223 3456 1234 9876'});
        is( $LineItemPaymentDCC->get('CreditCardNumber'), '1223345612349876',
            'Creditcardnumber Update - cutting out non-numbers (2)');

        $LineItemPaymentDCC->set({'CVV' => '321'});
        is( $LineItemPaymentDCC->get('CVV'), '321', 'CVV Update' );
    }

    # EPG-23749: Data too long for column 'creditcardnumber'
    # The following tests will succeed if the column
    # 'creditcardnumber' is set to varchar(96).
    my %hCreditCardNumbers = (
        AmericanExpress_15          => '378282246310005',
        AmericanExpressCorporate_15 => '378734493671000',
        AustralianBankCard_16       => '5610591081018250',
        DinersClub_14               => '30569309025904',
        Discover_16                 => '6011111111111117',
        JCB_16                      => '3530111333300000',
        MasterCard_16               => '5555555555554444',
        Visa_16                     => '4111111111111111',
        Visa_13                     => '4222222222222',
        Laser_19                    => '6304990000000000044',
    );

    use constant LONG_ENCRYPTED_LENGTH  => 96;
    use constant CREDITCARD_STANDARD_LENGTH => 16;
    while (my ($CreditCardIssuer, $CreditCardNumber) = each %hCreditCardNumbers) {
        my $LineItemPaymentDCC = $Container->insertChild( 'LineItemPaymentDCC', {
                'Alias'             => '08/15 ' . $CreditCardIssuer,
                'CreditCardType'    => $CreditCardIssuer,
                'CreditCardNumber'  => $CreditCardNumber,
                'LineItemContainer' => $Container,
            }
        );

        my $SQL = 'SELECT creditcardnumber FROM lineitempaymentdcc WHERE lineitempaymentdccid = ?';
        my $aaRows = $dbi->execute( $SQL, [ $LineItemPaymentDCC->id ] );
        ok( exists $aaRows->[0]->[0], 'encrypted number ' . $CreditCardNumber . ' exists' );
        if (length $CreditCardNumber > CREDITCARD_STANDARD_LENGTH) {
            is( length $aaRows->[0]->[0], LONG_ENCRYPTED_LENGTH,
                'check length of encrypted credit-card number ' . $CreditCardNumber .
                ' (length > ' . CREDITCARD_STANDARD_LENGTH . ')' );
        } else {
            is( length $aaRows->[0]->[0], SHORT_ENCRYPTED_LENGTH,
                'check length of encrypted credit-card number ' . $CreditCardNumber .
                ' (length <= ' . CREDITCARD_STANDARD_LENGTH . ')' );
        }
    }

    # cleanup
    ok( Cleanup(), 'cleanup' );

    done_testing();
    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );

    $Folder->insertChild(
        'Address', {
            'Alias'   => 'O1',
            'Street'  => 'Leutragraben',
            'City'    => 'Jena',
            'Zipcode' => '07740',
            'Company' => 'epages',
        }
    );

    return 1;
}

RunScript( Sub => \&Main, Debug => 1 );
