use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use DE_EPAGES::UPS::API::XML::Builder::ShipmentBuilder;
use Readonly;
use Test::More;
use Test::Exception;
use Data::Dumper;

Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitLongLines)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001');## no critic (ProhibitPathConcatenation)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($ShippingLabelUPS->get('Order'), 'DE_EPAGES::Order::API::Object::CustomerOrder');
                isa_ok($DataCollectors, 'DE_EPAGES::UPS::API::DataCollectors');
                my $ShipmentBuilder = DE_EPAGES::UPS::API::XML::Builder::ShipmentBuilder->new($DataCollectors);
                testDescription($ShipmentBuilder);
                testService($ShipmentBuilder);
                # caused by the negative tests, the "DataCollectors" object  must be reloaded
                $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                $ShipmentBuilder = DE_EPAGES::UPS::API::XML::Builder::ShipmentBuilder->new($DataCollectors);
                testShipTo($ShipmentBuilder);
                testShipFrom($ShipmentBuilder);
                testShipper($ShipmentBuilder);
                testPackage($ShipmentBuilder);
                testPaymentInformation($ShipmentBuilder);
                testAssamble($ShipmentBuilder);
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}
## no critic (ProhibitPrivateSubroutineUse)
sub testDescription {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_description()->{'Description'},'ShippingLabelUPS Description','call _Description with valid Data');
}
sub testTaxIdentificationNumber {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_taxIdentificationNumber()->{'TaxIdentificationNumber'},'123456','call _TaxIdentificationNumber with valid Data');
}
sub testShipperNumber {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_shipperNumber()->{'ShipperNumber'},'ShipperNumber','call _ShipperNumber with valid Data');
}

sub testService {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_service()->{'Service'}->{'Code'},'11','call _Service with valid Data');
}

sub testShipTo {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_shipTo()->{'ShipTo'}->{'Name'},'Max Mustermann','call _ShipTo()-> Name with valid Data');
    is ($ShipmentBuilder->_shipTo()->{'ShipTo'}->{'Address'}->{'City'},'Neustadt','call _ShipTo()->Address->City with valid Data');
}

sub testShipFrom {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_shipFrom()->{'ShipFrom'}->{'Name'},'ShopFirstName ShopLastName','call _ShipFrom()-> Name with valid Data');
    is ($ShipmentBuilder->_shipFrom()->{'ShipFrom'}->{'Address'}->{'City'},'ShopCity','call _ShipFrom()->Address->City with valid Data');
}

sub testShipper {
    my ($ShipmentBuilder) = @_;
    is ($ShipmentBuilder->_shipper()->{'Shipper'}->{'Name'},'ShopFirstName ShopLastName','call _Shipper()-> Name with valid Data');
    is ($ShipmentBuilder->_shipper()->{'Shipper'}->{'Address'}->{'City'},'ShopCity','call _Shipper()->Address->City with valid Data');
    is ($ShipmentBuilder->_shipper()->{'Shipper'}->{'TaxIdentificationNumber'},'123456789', 'call _Shipper()->TaxIdentificationNumber');
    is ($ShipmentBuilder->_shipper()->{'Shipper'}->{'ShipperNumber'},'ShipperNumber', 'call _Shipper()->ShipperNumber');
}

sub testPackage {
    my ($ShipmentBuilder) = @_;
    $ShipmentBuilder->_package();
    is (scalar keys %{$ShipmentBuilder->_package()},1,'call _Package() number of subitems');
    is (scalar keys %{$ShipmentBuilder->_package()->{'Package'}},4,'call _Package() number of subitems/Package');
}

sub testPaymentInformation {
    my ($ShipmentBuilder) = @_;
    my $hPaymentInformation = $ShipmentBuilder->_paymentInformation();
    is ($hPaymentInformation->{'PaymentInformation'}->{'ShipmentCharge'}->{'BillShipper'}->{'AccountNumber'},'ShipperNumber','call _PaymentInformation() PaymentInformation/ShipmentCharge/AccountNumber');
    is ($hPaymentInformation->{'PaymentInformation'}->{'ShipmentCharge'}->{'Type'},'01','call _PaymentInformation() PaymentInformation/ShipmentCharge/Type}');
}
sub testAssamble {
    my ($ShipmentBuilder) = @_;
    my $assemble = $ShipmentBuilder->assemble();
    is (scalar keys %{$assemble->{'Shipment'}},8,'call assemble() number of subItems');
}
RunScript( Sub => \&Main );

1;